#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-03-24
#
# only useful for Cisco WSA appliances
#
# 2023-03-06: fixed: IndexError in parse function
# 2024-04-18: fixed bandwidth values to bytes/s (kb/s), see MIB

from typing import NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    CheckResult,
    DiscoveryResult,
    HostLabelGenerator,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    HostLabel,
    SNMPTree,
    Service,
    check_levels,
    contains,
    register,
    render,
)


class CiscoAsyncosBandwidth(NamedTuple):
    bandwidth_now: int
    bandwidth_hour: int
    bandwidth_day: int


def host_label_cisco_asyncos(section: int) -> HostLabelGenerator:
    """Host label function
    Labels:
        cmk/os_family:
            This label is set to "AsyncOS" for all Cisco Ironport devices
    """
    yield HostLabel(name="cmk/os_family", value="AsyncOS")


# [['0', '0', '247']]
def parse_cisco_asyncos_bandwidth(string_table: StringTable) -> CiscoAsyncosBandwidth | None:
    try:
        return CiscoAsyncosBandwidth(
            bandwidth_now=int(string_table[0][0]) * 1000 / 8,  # kbits/s -> bytes/s
            bandwidth_hour=int(string_table[0][1]) * 1000 / 8,
            bandwidth_day=int(string_table[0][2]) * 1000 / 8,
        )
    except IndexError:
        return


def discovery_cisco_asyncos_bandwidth(section: CiscoAsyncosBandwidth) -> DiscoveryResult:
    yield Service()


def check_cisco_asyncos_bandwidth(params, section: CiscoAsyncosBandwidth) -> CheckResult:
    yield from check_levels(
        value=section.bandwidth_now,
        label='Current bandwidth',
        # levels_upper=params.get('upper', None),  # not implemented
        # levels_lower=params.get('lower', None),  # not implemented
        metric_name='bandwidth_now',
        render_func=render.networkbandwidth,
    )
    yield from check_levels(
        value=section.bandwidth_hour,
        label='Last hour',
        render_func=render.networkbandwidth
    )
    yield from check_levels(
        value=section.bandwidth_day,
        label='Last day',
        render_func=render.networkbandwidth
    )


register.snmp_section(
    name='cisco_asyncos_bandwidth',
    parse_function=parse_cisco_asyncos_bandwidth,
    host_label_function=host_label_cisco_asyncos,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.15497.1.2.3.7.4',  # ASYNCOSWEBSECURITYAPPLIANCE-MIB::proxyRecentBandWTotPerf
        oids=[
            '1',  # cacheBwidthTotalNow (kb/s -> 1000 bits/s)
            '3',  # cacheBwidthTotal1hrMean (kb/s)
            '5',  # cacheBwidthTotal1dayMean (kb/s)
        ]
    ),
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_bandwidth',
    service_name='Bandwidth',
    discovery_function=discovery_cisco_asyncos_bandwidth,
    check_function=check_cisco_asyncos_bandwidth,
    check_default_parameters={},
)
