#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-03-24
#
# only useful for Cisco WSA appliances
#

from typing import NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    CheckResult,
    DiscoveryResult,
    HostLabelGenerator,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    HostLabel,
    Metric,
    Result,
    SNMPTree,
    Service,
    State,
    contains,
    register,
)


class CiscoAsyncosCache(NamedTuple):
    cache_hits_now: int
    cache_misses_now: int
    cache_hit_resptime_now: int
    cache_miss_resptime_now: int
    cache_total_resptime: int


def host_label_cisco_asyncos(section: int) -> HostLabelGenerator:
    """Host label function
    Labels:
        cmk/os_family:
            This label is set to "AsyncOS" for all Cisco Ironport devices
    """
    yield HostLabel(name="cmk/os_family", value="AsyncOS")


# [['0', '0', '0', '0', '0']]
def parse_cisco_asyncos_cache(string_table: StringTable) -> CiscoAsyncosCache | None:
    try:
        return CiscoAsyncosCache(
            cache_hits_now=int(string_table[0][0]),
            cache_misses_now=int(string_table[0][1]),
            cache_hit_resptime_now=int(string_table[0][2]),
            cache_miss_resptime_now=int(string_table[0][3]),
            cache_total_resptime=int(string_table[0][4]),
        )
    except IndexError:
        return


def discovery_cisco_asyncos_cache(section: CiscoAsyncosCache) -> DiscoveryResult:
    yield Service()


def check_cisco_asyncos_cache(params, section: CiscoAsyncosCache) -> CheckResult:
    yield Result(
        state=State.OK,
        summary=f'Cache Stats current {section.cache_hits_now} Hits '
                f'({section.cache_hit_resptime_now}ms response time), '
                f'{section.cache_misses_now} Misses '
                f'({section.cache_miss_resptime_now}ms response time) and '
                f'{section.cache_total_resptime}ms total response time'
    )

    yield Metric(name='cache_hits_now', value=section.cache_hits_now)
    yield Metric(name='cache_misses_now', value=section.cache_misses_now)
    yield Metric(name='cache_hit_resptime_now', value=section.cache_hit_resptime_now)
    yield Metric(name='cache_miss_resptime_now', value=section.cache_miss_resptime_now)
    yield Metric(name='cache_total_resptime', value=section.cache_total_resptime)


register.snmp_section(
    name='cisco_asyncos_cache',
    parse_function=parse_cisco_asyncos_cache,
    host_label_function=host_label_cisco_asyncos,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.15497.1.2.3.7',  # ASYNCOSWEBSECURITYAPPLIANCE-MIB::proxyRecentPerf
        oids=[
            '5.1',  # cacheHitsNow
            '6.1',  # cacheMissesNow
            '7.1',  # cacheHitRespTimeNow
            '8.1',  # cacheMissRespTimeNow
            '9.1',  # cacheTotalRespTimeNow
        ]
    ),
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_cache',
    service_name='Cache Stats',
    discovery_function=discovery_cisco_asyncos_cache,
    check_function=check_cisco_asyncos_cache,
    check_default_parameters={},
)
