#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-03-24
#
# only use full for Cisco WSA appliances
#
from typing import NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    CheckResult,
    DiscoveryResult,
    HostLabelGenerator,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    HostLabel,
    Metric,
    Result,
    SNMPTree,
    Service,
    State,
    contains,
    register,
)


class CiscoAsyncosConn(NamedTuple):
    cache_client_idle_conns: int
    cache_client_total_conns: int
    cache_client_max_conns: int


def host_label_cisco_asyncos(section: int) -> HostLabelGenerator:
    """Host label function
    Labels:
        cmk/os_family:
            This label is set to "AsyncOS" for all Cisco Ironport devices
    """
    yield HostLabel(name="cmk/os_family", value="AsyncOS")


# [['0', '5', '66277']]
def parse_cisco_asyncos_conn(string_table: StringTable) -> CiscoAsyncosConn | None:
    try:
        return CiscoAsyncosConn(
            cache_client_idle_conns=int(string_table[0][0]),
            cache_client_total_conns=int(string_table[0][1]),
            cache_client_max_conns=int(string_table[0][2]),
        )
    except IndexError:
        return


def discovery_cisco_asyncos_conn(section: CiscoAsyncosConn) -> DiscoveryResult:
    yield Service()


def check_cisco_asyncos_conn(params, section: CiscoAsyncosConn) -> CheckResult:
    yield Result(
        state=State.OK,
        summary=f'{section.cache_client_total_conns} Active Connections - '
                f'{section.cache_client_idle_conns} Idle Connections - Maximum Connections was '
                f'{section.cache_client_max_conns}'
    )

    yield Metric(name='cacheClientIdleConns', value=section.cache_client_idle_conns)
    yield Metric(name='cacheClientTotalConns', value=section.cache_client_total_conns)
    yield Metric(name='cacheClientMaxConns', value=section.cache_client_max_conns)


register.snmp_section(
    name='cisco_asyncos_conn',
    parse_function=parse_cisco_asyncos_conn,
    host_label_function=host_label_cisco_asyncos,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.15497.1.2.3.2',  # ASYNCOSWEBSECURITYAPPLIANCE-MIB::proxyClientSidePerf
        oids=[
            '7',  # cacheClientIdleConns
            '8',  # cacheClientTotalConns
            '9',  # cacheClientMaxConns
        ]
    ),
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_conn',
    service_name='Connection Stats',
    discovery_function=discovery_cisco_asyncos_conn,
    check_function=check_cisco_asyncos_conn,
    check_default_parameters={},
)
