#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-03-08
#
# monitors status Cisco IronPort Appliances (ESA) DNS requests
# Tested with: C380, C370
#
# 2021-03-24: rewrite for CMK 2.0
# 2024-04-18: changed to check_levels
#             added upper levels
#             added metrics definition

#
# .1.3.6.1.4.1.15497.1.1.1.15.0 = Gauge32: 0
# .1.3.6.1.4.1.15497.1.1.1.16.0 = Gauge32: 0
#
# ASYNCOS-MAIL-MIB::outstandingDNSRequests.0 = Gauge32: 0
# ASYNCOS-MAIL-MIB::pendingDNSRequests.0 = Gauge32: 0
#
#
from typing import NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    CheckResult,
    DiscoveryResult,
    HostLabelGenerator,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    HostLabel,
    SNMPTree,
    Service,
    check_levels,
    contains,
    register,
)


class CiscoAsyncosDns(NamedTuple):
    outstanding_dns_requests: int
    pending_dns_requests: int


def host_label_cisco_asyncos(section: int) -> HostLabelGenerator:
    """Host label function
    Labels:
        cmk/os_family:
            This label is set to "AsyncOS" for all Cisco Ironport devices
    """
    yield HostLabel(name="cmk/os_family", value="AsyncOS")


# [['0', '0']]
def parse_cisco_asyncos_dns(string_table: StringTable) -> CiscoAsyncosDns | None:
    try:
        return CiscoAsyncosDns(
            outstanding_dns_requests=int(string_table[0][0]),
            pending_dns_requests=int(string_table[0][1]),
        )
    except IndexError:
        return


def discovery_cisco_asyncos_dns(section: CiscoAsyncosDns) -> DiscoveryResult:
    yield Service()


def check_cisco_asyncos_dns(params, section: CiscoAsyncosDns) -> CheckResult:
    yield from check_levels(
        value=section.outstanding_dns_requests,
        label='Outstanding requests',
        metric_name='outstandingDNSRequests',
        render_func=lambda v: f'{v}',
        levels_upper=params.get('levels_outstanding')
    )
    yield from check_levels(
        value=section.pending_dns_requests,
        label='Pending requests',
        metric_name='pendingDNSRequests',
        render_func=lambda v: f'{v}',
        levels_upper=params.get('levels_pending')
    )


register.snmp_section(
    name='cisco_asyncos_dns',
    parse_function=parse_cisco_asyncos_dns,
    host_label_function=host_label_cisco_asyncos,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.15497.1.1.1',  # ASYNCOS-MAIL-MIB
        oids=[
            '15',  # outstandingDNSRequests
            '16',  # pendingDNSRequests
        ]
    ),
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_dns',
    service_name='DNS requests',
    discovery_function=discovery_cisco_asyncos_dns,
    check_function=check_cisco_asyncos_dns,
    check_default_parameters={},
    check_ruleset_name='cisco_asyncos_dns'
)
