#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-02-10

#
# monitors status Cisco IronPort Appliances (ESA/SMA/WSA) fan sensors
# Tested with: C380, M380, C370, M670, S370
#
# 2021-03-21: rewrite for CMK 2.0
#
# sample snmpwalk
# .1.3.6.1.4.1.15497.1.1.1.10.1.1.1 = INTEGER: 1
# .1.3.6.1.4.1.15497.1.1.1.10.1.1.2 = INTEGER: 2
# .1.3.6.1.4.1.15497.1.1.1.10.1.1.3 = INTEGER: 3
# .1.3.6.1.4.1.15497.1.1.1.10.1.1.4 = INTEGER: 4
# .1.3.6.1.4.1.15497.1.1.1.10.1.2.1 = Gauge32: 3600
# .1.3.6.1.4.1.15497.1.1.1.10.1.2.2 = Gauge32: 3600
# .1.3.6.1.4.1.15497.1.1.1.10.1.2.3 = Gauge32: 3600
# .1.3.6.1.4.1.15497.1.1.1.10.1.2.4 = Gauge32: 3480
# .1.3.6.1.4.1.15497.1.1.1.10.1.3.1 = STRING: FAN 1
# .1.3.6.1.4.1.15497.1.1.1.10.1.3.2 = STRING: FAN 2
# .1.3.6.1.4.1.15497.1.1.1.10.1.3.3 = STRING: FAN 3
# .1.3.6.1.4.1.15497.1.1.1.10.1.3.4 = STRING: FAN 4
#
# ASYNCOS-MAIL-MIB::fanIndex.1 = INTEGER: 1
# ASYNCOS-MAIL-MIB::fanIndex.2 = INTEGER: 2
# ASYNCOS-MAIL-MIB::fanIndex.3 = INTEGER: 3
# ASYNCOS-MAIL-MIB::fanIndex.4 = INTEGER: 4
# ASYNCOS-MAIL-MIB::fanRPMs.1 = Gauge32: 3600
# ASYNCOS-MAIL-MIB::fanRPMs.2 = Gauge32: 3600
# ASYNCOS-MAIL-MIB::fanRPMs.3 = Gauge32: 3600
# ASYNCOS-MAIL-MIB::fanRPMs.4 = Gauge32: 3480
# ASYNCOS-MAIL-MIB::fanName.1 = STRING: FAN 1
# ASYNCOS-MAIL-MIB::fanName.2 = STRING: FAN 2
# ASYNCOS-MAIL-MIB::fanName.3 = STRING: FAN 3
# ASYNCOS-MAIL-MIB::fanName.4 = STRING: FAN 4
#

from typing import Dict, List

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    CheckResult,
    DiscoveryResult,
    HostLabelGenerator,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    HostLabel,
    SNMPTree,
    Service,
    check_levels,
    contains,
    register,
)


def host_label_cisco_asyncos(section: int) -> HostLabelGenerator:
    """Host label function
    Labels:
        cmk/os_family:
            This label is set to "AsyncOS" for all Cisco Ironport devices
    """
    yield HostLabel(name="cmk/os_family", value="AsyncOS")


#
# [[['6400', 'FAN 1'], ['6300', 'FAN 2'], ['6400', 'FAN 3'], ['6400', 'FAN 4'], ['6400', 'FAN 5'], ['6800', 'FAN 6']]]
#

def parse_cisco_asyncos_fan(string_table: List[StringTable]) -> Dict:
    sensors = {}
    for sensor in string_table[0]:
        try:
            value, item = sensor
            if item.startswith('FAN '):
                item = item.replace('FAN ', '')
            sensors.update({item: int(value)})
        except (ValueError, IndexError):
            pass
    return sensors


def discovery_cisco_asyncos_fan(section: Dict) -> DiscoveryResult:
    for key in section.keys():
        yield Service(item=key)


def check_cisco_asyncos_fan(item, params, section) -> CheckResult:
    try:
        value = section[item]
    except KeyError:
        return

    yield from check_levels(
        value=value,
        label='Speed',
        levels_lower=params.get('lower', None),
        levels_upper=params.get('upper', None),
        metric_name='fan' if params.get('output_metrics') else None,
        render_func=lambda v: f'{str(v)} RPM',
    )


register.snmp_section(
    name='cisco_asyncos_fan',
    parse_function=parse_cisco_asyncos_fan,
    host_label_function=host_label_cisco_asyncos,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.15497.1.1.1.10.1',  # ASYNCOS-MAIL-MIB::fanEntry
            oids=[
                '2',  # fanRPMs
                '3',  # fanName
            ]
        ),
    ],
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_fan',
    service_name='Fan %s',
    discovery_function=discovery_cisco_asyncos_fan,
    check_function=check_cisco_asyncos_fan,
    check_default_parameters={},
    check_ruleset_name='hw_fans'
)
