#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-03-08
#
# monitors status Cisco IronPort Appliances (ESA) oldest message age
# Tested with: C380, C370
#
# 2021-03-24: rewrite for CMK 2.0
#
# .1.3.6.1.4.1.15497.1.1.1.14.0 = Gauge32: 258454
#
# ASYNCOS-MAIL-MIB::oldestMessageAge.0 = Gauge32: 259706
#

from typing import NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    CheckResult,
    DiscoveryResult,
    HostLabelGenerator,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    HostLabel,
    SNMPTree,
    Service,
    check_levels,
    contains,
    register,
    render,
)


class CiscoAsyncosMessageage(NamedTuple):
    message: int


def host_label_cisco_asyncos(section: int) -> HostLabelGenerator:
    """Host label function
    Labels:
        cmk/os_family:
            This label is set to "AsyncOS" for all Cisco Ironport devices
    """
    yield HostLabel(name="cmk/os_family", value="AsyncOS")


# [['259297']]
def parse_cisco_asyncos_messageage(string_table: StringTable) -> CiscoAsyncosMessageage | None:
    try:
        return CiscoAsyncosMessageage(
            message=int(string_table[0][0]),
        )
    except IndexError:
        return


def discovery_cisco_asyncos_messageage(section: CiscoAsyncosMessageage) -> DiscoveryResult:
    yield Service()


def check_cisco_asyncos_messageage(params, section: CiscoAsyncosMessageage) -> CheckResult:
    yield from check_levels(
        value=section.message,
        label='Oldest messages',
        metric_name='message_age',
        levels_upper=params.get('upper', None),
        render_func=render.timespan,
    )


register.snmp_section(
    name='cisco_asyncos_messageage',
    parse_function=parse_cisco_asyncos_messageage,
    host_label_function=host_label_cisco_asyncos,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.15497.1.1.1',  # ASYNCOS-MAIL-MIB
        oids=[
            '14',  # oldestMessageAge
        ]
    ),
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_messageage',
    service_name='Message age',
    discovery_function=discovery_cisco_asyncos_messageage,
    check_function=check_cisco_asyncos_messageage,
    check_default_parameters={},
)
