#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-02-10
#
# monitors status power supplies of Cisco IronPort Appliances (ESA/SMA/WSA)
# Tested with: C380, M380, C370, M670, S370
#
# 2021-03-21: rewrite for CMK 2.0
#
# sample snmpwalk
# .1.3.6.1.4.1.15497.1.1.1.8.1.1.1 = INTEGER: 1
# .1.3.6.1.4.1.15497.1.1.1.8.1.1.2 = INTEGER: 2
# .1.3.6.1.4.1.15497.1.1.1.8.1.2.1 = INTEGER: 2
# .1.3.6.1.4.1.15497.1.1.1.8.1.2.2 = INTEGER: 2
# .1.3.6.1.4.1.15497.1.1.1.8.1.3.1 = INTEGER: 1
# .1.3.6.1.4.1.15497.1.1.1.8.1.3.2 = INTEGER: 1
# .1.3.6.1.4.1.15497.1.1.1.8.1.4.1 = STRING: PS 1
# .1.3.6.1.4.1.15497.1.1.1.8.1.4.2 = STRING: PS 2
#
# ASYNCOS-MAIL-MIB::powerSupplyIndex.1 = INTEGER: 1
# ASYNCOS-MAIL-MIB::powerSupplyIndex.2 = INTEGER: 2
# ASYNCOS-MAIL-MIB::powerSupplyStatus.1 = INTEGER: powerSupplyHealthy(2)
# ASYNCOS-MAIL-MIB::powerSupplyStatus.2 = INTEGER: powerSupplyHealthy(2)
# ASYNCOS-MAIL-MIB::powerSupplyRedundancy.1 = INTEGER: powerSupplyRedundancyOK(1)
# ASYNCOS-MAIL-MIB::powerSupplyRedundancy.2 = INTEGER: powerSupplyRedundancyOK(1)
# ASYNCOS-MAIL-MIB::powerSupplyName.1 = STRING: PS 1
# ASYNCOS-MAIL-MIB::powerSupplyName.2 = STRING: PS 2
#

from typing import Dict, List, NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    CheckResult,
    DiscoveryResult,
    HostLabelGenerator,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    HostLabel,
    Result,
    SNMPTree,
    Service,
    State,
    contains,
    register,
)


class CiscoAsyncosPowersupply(NamedTuple):
    redundancy: State
    redundancy_readable: str
    state: State
    state_readable: str


def host_label_cisco_asyncos(section: int) -> HostLabelGenerator:
    """Host label function
    Labels:
        cmk/os_family:
            This label is set to "AsyncOS" for all Cisco Ironport devices
    """
    yield HostLabel(name="cmk/os_family", value="AsyncOS")


#
# [[['2', '1', 'PS 1'], ['2', '1', 'PS 2']]]
#
def parse_cisco_asyncos_power(string_table: List[StringTable]) -> dict:
    def get_ps_state(st: str):
        state = {
            '1': (State.WARN, 'not installed'),
            '2': (State.OK, 'healthy'),
            '3': (State.CRIT, 'no AC'),
            '4': (State.CRIT, 'faulty'),
        }
        return state.get(st)

    def get_ps_redundancy(st: str):
        redundancy = {
            '1': (State.OK, 'OK'),
            '2': (State.CRIT, 'redundancy lost'),
        }
        return redundancy.get(st)

    sensors = {}

    for sensor in string_table[0]:
        try:
            state, redundancy, item = sensor
            if item.startswith('PS '):
                item = item.replace('PS ', '')

            state, state_readable = get_ps_state(state)
            redundancy, redundancy_readable = get_ps_redundancy(redundancy)

            sensors.update({item: CiscoAsyncosPowersupply(
                redundancy=redundancy,
                redundancy_readable=redundancy_readable,
                state=state,
                state_readable=state_readable,
            )})
        except (ValueError, IndexError):
            pass
    return sensors


#
# {
#  '1': CiscoAsyncosPowersupply(redundancy=<State.OK: 0>, state=<State.OK: 0>, redundancy_readable='OK', state_readable='healthy'),
#  '2': CiscoAsyncosPowersupply(redundancy=<State.OK: 0>, state=<State.OK: 0>, redundancy_readable='OK', state_readable='healthy')
# }
#

def discovery_cisco_asyncos_power(section: Dict) -> DiscoveryResult:
    for key in section.keys():
        yield Service(item=key)


def check_cisco_asyncos_power(item, section) -> CheckResult:
    try:
        sensor = section[item]

        yield Result(state=sensor.state, summary=f'Status: {sensor.state_readable}')
        yield Result(state=sensor.redundancy, summary=f'Redundancy: {sensor.redundancy_readable}')

    except KeyError:
        pass


register.snmp_section(
    name='cisco_asyncos_power',
    parse_function=parse_cisco_asyncos_power,
    host_label_function=host_label_cisco_asyncos,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.15497.1.1.1.8.1',  # ASYNCOS-MAIL-MIB::powerSupplyEntry
            oids=[
                '2',  # powerSupplyStatus
                '3',  # powerSupplyRedundancy
                '4',  # powerSupplyName
            ]
        ),
    ],
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_power',
    service_name='Power Supply %s',
    discovery_function=discovery_cisco_asyncos_power,
    check_function=check_cisco_asyncos_power,
    # check_default_parameters={},
)
