#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-02-10
#
#  added wato, fixed inventory function, changed snmp scan function, changed item to 'work queue'
#
# 2021-03-24: rewrite for CMK 2.0
# 2024-04-18: fixed wrong queue states (ThX to alexander[dot]laengle[at]lgt[dot]com)
#             fixed levels for queue full not working (ThX to alexander[dot]laengle[at]lgt[dot]com)
#             changed state for not defined state from CRIT to UNKNOWN
#             a bit of clean up

from typing import NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    CheckResult,
    DiscoveryResult,
    HostLabelGenerator,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    HostLabel,
    Result,
    SNMPTree,
    Service,
    State,
    check_levels,
    contains,
    register,
)


class CiscoAsyncosQueue(NamedTuple):
    state: State
    state_readable: str
    work_queue_messages: int | None


def get_state_readable(st: str) -> str:
    states = {
        '1': 'enough',
        '2': 'near full',
        '3': 'full',
    }
    return states.get(st, st)


def get_cmk_state(st: str) -> State:
    states = {
        '1': State.OK,
        '2': State.WARN,
        '3': State.CRIT
    }
    return states.get(st, State.UNKNOWN)


def host_label_cisco_asyncos(section: int) -> HostLabelGenerator:
    """Host label function
    Labels:
        cmk/os_family:
            This label is set to "AsyncOS" for all Cisco Ironport devices
    """
    yield HostLabel(name="cmk/os_family", value="AsyncOS")


# [['1', '3451']]
def parse_cisco_asyncos_queue(string_table: StringTable) -> CiscoAsyncosQueue | None:
    try:
        queue_availability_status, work_queue_messages = string_table[0]
    except IndexError:
        return

    return CiscoAsyncosQueue(
        state=get_cmk_state(queue_availability_status),
        state_readable=get_state_readable(queue_availability_status),
        work_queue_messages=int(work_queue_messages) if work_queue_messages.isdigit() else None
    )


def discovery_cisco_asyncos_queue(section: CiscoAsyncosQueue) -> DiscoveryResult:
    yield Service()


def check_cisco_asyncos_queue(params, section: CiscoAsyncosQueue) -> CheckResult:
    yield from check_levels(
        value=section.work_queue_messages,
        label='Messages in work queue',
        levels_upper=params.get('levels', None),
        metric_name='work_queue_size',
    )
    yield Result(state=section.state, notice=f'Queue space: {section.state_readable}')


register.snmp_section(
    name='cisco_asyncos_queue',
    parse_function=parse_cisco_asyncos_queue,
    host_label_function=host_label_cisco_asyncos,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.15497.1.1.1',  # ASYNCOS-MAIL-MIB::asyncOSMailObjects
        oids=[
            '5',  # queueAvailabilityStatus
            '11'  # workQueueMessages
        ]
    ),
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_queue',
    service_name='Queue',
    discovery_function=discovery_cisco_asyncos_queue,
    check_function=check_cisco_asyncos_queue,
    check_default_parameters={'levels': (50, 100)},
    check_ruleset_name='cisco_asyncos_queue',
)
