#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-02-10
#
#
# monitors status raid arrays of Cisco IronPort Appliances (ESA/SMA/WSA)
# Tested with: C380, M380, C370, M670, S370
#
# 2021-03-24: rewrite for CMK 2.0
#
# OMD[cmk16x]:~$ snmpwalk  -v2c -c C370-MAIL localhost -m ASYNCOS-MAIL-MIB -ObentU raidTable
# .1.3.6.1.4.1.15497.1.1.1.18.1.1.1 = INTEGER: 1
# .1.3.6.1.4.1.15497.1.1.1.18.1.1.2 = INTEGER: 2
# .1.3.6.1.4.1.15497.1.1.1.18.1.2.1 = INTEGER: 1
# .1.3.6.1.4.1.15497.1.1.1.18.1.2.2 = INTEGER: 1
# .1.3.6.1.4.1.15497.1.1.1.18.1.3.1 = STRING: Drive 0
# .1.3.6.1.4.1.15497.1.1.1.18.1.3.2 = STRING: Drive 1
# .1.3.6.1.4.1.15497.1.1.1.18.1.4.1 = STRING: No Errors
# .1.3.6.1.4.1.15497.1.1.1.18.1.4.2 = STRING: No Errors
# 
# OMD[cmk16x]:~$ snmpwalk  -v2c -c C370-MAIL localhost -m ASYNCOS-MAIL-MIB  raidTable
# ASYNCOS-MAIL-MIB::raidIndex.1 = INTEGER: 1
# ASYNCOS-MAIL-MIB::raidIndex.2 = INTEGER: 2
# ASYNCOS-MAIL-MIB::raidStatus.1 = INTEGER: driveHealthy(1)
# ASYNCOS-MAIL-MIB::raidStatus.2 = INTEGER: driveHealthy(1)
# ASYNCOS-MAIL-MIB::raidID.1 = STRING: Drive 0
# ASYNCOS-MAIL-MIB::raidID.2 = STRING: Drive 1
# ASYNCOS-MAIL-MIB::raidLastError.1 = STRING: No Errors
# ASYNCOS-MAIL-MIB::raidLastError.2 = STRING: No Errors
#
from typing import Mapping, List, NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    CheckResult,
    DiscoveryResult,
    HostLabelGenerator,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    HostLabel,
    Result,
    SNMPTree,
    Service,
    State,
    contains,
    register,
)


class CiscoAsyncosRaid(NamedTuple):
    state: State
    state_readable: str
    last_error: str


def get_cmk_state(st: str) -> State:
    states = {
        '1': State.OK,
        '2': State.CRIT,
        '3': State.WARN
    }
    return states.get(st, State.CRIT)


def get_state_readable(st: str) -> str:
    states = {
        '1': 'healthy',
        '2': 'failure',
        '3': 'rebuild',
    }
    return states.get(st, st)


def host_label_cisco_asyncos(section: int) -> HostLabelGenerator:
    """Host label function
    Labels:
        cmk/os_family:
            This label is set to "AsyncOS" for all Cisco Ironport devices
    """
    yield HostLabel(name="cmk/os_family", value="AsyncOS")


# [[
#   ['1', 'Drive 0', 'No Errors'],
#   ['2', 'Drive 1', 'UNKNOWN'],
#   ['1', 'Drive 2', 'No Errors'],
#   ['1', 'Drive 3', 'No Errors'],
#   ['1', 'Drive 4', 'No Errors'],
#   ['1', 'Drive 5', 'No Errors']
#  ]]
def parse_cisco_asyncos_raid(string_table: List[StringTable]) -> Mapping[str, CiscoAsyncosRaid]:
    arrays = {}
    for raid_status, raid_id, raid_last_error in string_table[0]:
        arrays.update({raid_id: CiscoAsyncosRaid(
            state=get_cmk_state(raid_status),
            state_readable=get_state_readable(raid_status),
            last_error=raid_last_error
        )})
    return arrays


# {
#  'Drive 0': CiscoAsyncosRaid(state=<State.OK: 0>, status_readable='Ok', lasterror='No Errors'),
#  'Drive 1': CiscoAsyncosRaid(state=<State.CRIT: 2>, status_readable='failure', lasterror='UNKNOWN'),
#  'Drive 2': CiscoAsyncosRaid(state=<State.OK: 0>, status_readable='Ok', lasterror='No Errors'),
#  'Drive 3': CiscoAsyncosRaid(state=<State.OK: 0>, status_readable='Ok', lasterror='No Errors'),
#  'Drive 4': CiscoAsyncosRaid(state=<State.OK: 0>, status_readable='Ok', lasterror='No Errors'),
#  'Drive 5': CiscoAsyncosRaid(state=<State.OK: 0>, status_readable='Ok', lasterror='No Errors')
# }

def discovery_cisco_asyncos_raid(section: Mapping[str, CiscoAsyncosRaid]) -> DiscoveryResult:
    yield from (Service(item=item) for item in section.keys())


def check_cisco_asyncos_raid(item, section: Mapping[str, CiscoAsyncosRaid]) -> CheckResult:
    try:
        yield Result(
            state=section[item].state,
            summary=f'RAID status: {section[item].state}, last error: {section[item].last_error}',
        )
    except KeyError:
        pass


register.snmp_section(
    name='cisco_asyncos_raid',
    parse_function=parse_cisco_asyncos_raid,
    host_label_function=host_label_cisco_asyncos,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.15497.1.1.1.18.1',  # ASYNCOS-MAIL-MIB::raidEntry
            oids=[
                '2',  # raidStatus
                '3',  # raidID
                '4',  # raidLastError
            ]
        ),
    ],
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_raid',
    service_name='RAID %s',
    discovery_function=discovery_cisco_asyncos_raid,
    check_function=check_cisco_asyncos_raid,
)
