#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-03-08
#
# monitors status Cisco IronPort Appliances (ESA) system resources
# Tested with: C380, C370
#
# 2021-03-25: rewrite for CMK2.0
#
# .1.3.6.1.4.1.15497.1.1.1.6.0 = INTEGER: 1
#
# ASYNCOS-MAIL-MIB::resourceConservationReason.0 = INTEGER: noResourceConservation(1)
#
#
from typing import NamedTuple

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    CheckResult,
    DiscoveryResult,
    HostLabelGenerator,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    HostLabel,
    Result,
    SNMPTree,
    Service,
    State,
    contains,
    register,
)


def get_state_readable(st: str) -> str:
    states = {
        '1': 'none',
        '2': 'memory shortage',
        '3': 'queue space shortage',
        '4': 'queue Full',
    }
    return states.get(st, st)


def get_cmk_state(st: str) -> State:
    states = {
        '1': State.OK,
        '2': State.WARN,
        '3': State.WARN,
        '4': State.CRIT,
    }
    return states.get(st, State.CRIT)


class CiscoAsyncosResources(NamedTuple):
    state: State
    state_readable: str


def host_label_cisco_asyncos(section: int) -> HostLabelGenerator:
    """Host label function
    Labels:
        cmk/os_family:
            This label is set to "AsyncOS" for all Cisco Ironport devices
    """
    yield HostLabel(name="cmk/os_family", value="AsyncOS")


def parse_cisco_asyncos_bandwidth(string_table: StringTable) -> CiscoAsyncosResources | None:
    try:
        return CiscoAsyncosResources(
            state=get_cmk_state(string_table[0][0]),
            state_readable=get_state_readable(string_table[0][0])
        )
    except IndexError:
        return


def discovery_cisco_asyncos_resources(section: CiscoAsyncosResources) -> DiscoveryResult:
    yield Service()


def check_cisco_asyncos_resources(section: CiscoAsyncosResources) -> CheckResult:
    yield Result(state=section.state, notice=f'Resource conservation {section.state_readable}')


register.snmp_section(
    name='cisco_asyncos_resources',
    parse_function=parse_cisco_asyncos_bandwidth,
    host_label_function=host_label_cisco_asyncos,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.15497.1.1.1',  # ASYNCOS-MAIL-MIB
        oids=[
            '6',  # resourceConservationReason
        ]
    ),
    detect=contains('.1.3.6.1.2.1.1.1.0', 'AsyncOS'),
)

register.check_plugin(
    name='cisco_asyncos_resources',
    service_name='Resources',
    discovery_function=discovery_cisco_asyncos_resources,
    check_function=check_cisco_asyncos_resources,
)
